@@ -78,6 +78,17 @@ void vtkOpenGLGlyph3DHelper::GetShaderTemplate(
   shaders[vtkShader::Vertex]->SetSource(vtkGlyph3DVS);
 }
 
+//------------------------------------------------------------------------------
+void vtkOpenGLGlyph3DHelper::ReplaceShaderValues(
+  std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
+{
+  this->Superclass::ReplaceShaderValues(shaders, ren, actor);
+#ifdef GL_ES_VERSION_3_0
+  this->ReplaceShaderPointSize(shaders, ren, actor);
+#endif
+}
+
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderPositionVC(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
 {
@@ -101,6 +112,7 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderPositionVC(
   this->Superclass::ReplaceShaderPositionVC(shaders, ren, actor);
 }
 
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderColor(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
 {
@@ -142,11 +154,24 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderColor(
   // now handle scalar coloring
   if (!this->DrawingVertices)
   {
-    vtkShaderProgram::Substitute(FSSource, "//VTK::Color::Impl",
-      "//VTK::Color::Impl\n"
-      "  diffuseColor = diffuseIntensity * vertexColorVSOutput.rgb;\n"
-      "  ambientColor = ambientIntensity * vertexColorVSOutput.rgb;\n"
-      "  opacity = opacity * vertexColorVSOutput.a;");
+    if (actor->GetBackfaceProperty())
+    {
+      vtkShaderProgram::Substitute(FSSource, "//VTK::Color::Impl",
+        "//VTK::Color::Impl\n"
+        "  diffuseColor = (gl_FrontFacing) ? diffuseIntensity * vertexColorVSOutput.rgb : "
+        "diffuseIntensityBF * diffuseColorUniformBF;\n"
+        "  ambientColor = (gl_FrontFacing) ? ambientIntensity * vertexColorVSOutput.rgb : "
+        "ambientIntensityBF * ambientColorUniformBF;\n"
+        "  opacity = (gl_FrontFacing) ? opacity * vertexColorVSOutput.a : opacityUniformBF;");
+    }
+    else
+    {
+      vtkShaderProgram::Substitute(FSSource, "//VTK::Color::Impl",
+        "//VTK::Color::Impl\n"
+        "  diffuseColor = diffuseIntensity * vertexColorVSOutput.rgb;\n"
+        "  ambientColor = ambientIntensity * vertexColorVSOutput.rgb;\n"
+        "  opacity = opacity * vertexColorVSOutput.a;\n");
+    }
   }
 
   if (this->UsingInstancing)
@@ -167,6 +192,7 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderColor(
   this->Superclass::ReplaceShaderColor(shaders, ren, actor);
 }
 
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderNormal(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
 {
@@ -229,6 +255,7 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderClip(
   this->Superclass::ReplaceShaderClip(shaders, ren, actor);
 }
 
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderPicking(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer*, vtkActor*)
 {
@@ -243,6 +270,18 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderPicking(
   shaders[vtkShader::Fragment]->SetSource(FSSource);
 }
 
+//------------------------------------------------------------------------------
+void vtkOpenGLGlyph3DHelper::ReplaceShaderPointSize(
+  std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer*, vtkActor*)
+{
+  std::string VSSource = shaders[vtkShader::Vertex]->GetSource();
+  // Point size
+  vtkShaderProgram::Substitute(VSSource, "//VTK::PointSizeGLES30::Dec", "uniform float pointSize;");
+  vtkShaderProgram::Substitute(
+    VSSource, "//VTK::PointSizeGLES30::Impl", "gl_PointSize = pointSize;");
+  shaders[vtkShader::Vertex]->SetSource(VSSource);
+}
+
 void vtkOpenGLGlyph3DHelper::GlyphRender(vtkRenderer* ren, vtkActor* actor, vtkIdType numPts,
   std::vector<unsigned char>& colors, std::vector<float>& matrices,
   std::vector<float>& normalMatrices, std::vector<vtkIdType>& pickIds, vtkMTimeType pointMTime,
@@ -254,14 +293,13 @@ void vtkOpenGLGlyph3DHelper::GlyphRender(vtkRenderer* ren, vtkActor* actor, vtkI
   this->UsingInstancing = false;
 
   vtkHardwareSelector* selector = ren->GetSelector();
-
-  if (!selector && GLEW_ARB_instanced_arrays)
+  if (!selector && GLAD_GL_ARB_instanced_arrays)
   {
     // if there is no triangle, culling is useless.
-    // GLEW_ARB_gpu_shader5 is needed by the culling shader.
+    // GLAD_GL_ARB_gpu_shader5 is needed by the culling shader.
 #ifndef GL_ES_VERSION_3_0
-    if (this->CurrentInput->GetNumberOfPolys() <= 0 || !GLEW_ARB_gpu_shader5 ||
-      !GLEW_ARB_transform_feedback3)
+    if (this->CurrentInput->GetNumberOfPolys() <= 0 || !GLAD_GL_ARB_gpu_shader5 ||
+      !GLAD_GL_ARB_transform_feedback3)
     {
       culling = false;
     }
@@ -287,15 +325,22 @@ void vtkOpenGLGlyph3DHelper::GlyphRender(vtkRenderer* ren, vtkActor* actor, vtkI
 
   if (selecting_points)
   {
+#ifndef GL_ES_VERSION_3_0
     ostate->vtkglPointSize(6.0);
+#else
+    (void)ostate;
+#endif
     representation = GL_POINTS;
   }
-
-  bool draw_surface_with_edges =
-    (actor->GetProperty()->GetEdgeVisibility() && representation == VTK_SURFACE) && !selector;
+  int iEnd = vtkOpenGLPolyDataMapper::PrimitiveEnd;
+  if (selector && selector->GetFieldAssociation() == vtkDataObject::FIELD_ASSOCIATION_POINTS)
+  {
+    // when selecting points, the selection pass renders points at a larger size.
+    // so don't show vertices as they might conflict with the selection pass.
+    iEnd = vtkOpenGLPolyDataMapper::PrimitiveVertices;
+  }
   int numVerts = this->VBOs->GetNumberOfTuples("vertexMC");
-  for (int i = PrimitiveStart;
-       i < (draw_surface_with_edges ? PrimitiveEnd : PrimitiveTriStrips + 1); i++)
+  for (int i = PrimitiveStart; i < iEnd; i++)
   {
     this->DrawingVertices = i > PrimitiveTriStrips;
     if (this->Primitives[i].IBO->IndexCount)
@@ -336,7 +381,12 @@ void vtkOpenGLGlyph3DHelper::GlyphRender(vtkRenderer* ren, vtkActor* actor, vtkI
           }
           program->SetUniform3f("mapperIndex", selector->GetPropColorValue());
         }
-
+#ifdef GL_ES_VERSION_3_0
+        if (selecting_points)
+        {
+          program->SetUniformf("pointSize", 6.0);
+        }
+#endif
         glDrawRangeElements(mode, 0, static_cast<GLuint>(numVerts - 1),
           static_cast<GLsizei>(this->Primitives[i].IBO->IndexCount), GL_UNSIGNED_INT, nullptr);
       }
@@ -351,12 +401,6 @@ void vtkOpenGLGlyph3DHelper::SetMapperShaderParameters(
   vtkOpenGLHelper& cellBO, vtkRenderer* ren, vtkActor* actor)
 {
   this->Superclass::SetMapperShaderParameters(cellBO, ren, actor);
-
-  vtkHardwareSelector* selector = ren->GetSelector();
-  if (selector)
-  {
-    cellBO.Program->SetUniform3f("mapperIndex", selector->GetPropColorValue());
-  }
 }
 
 void vtkOpenGLGlyph3DHelper::GlyphRenderInstances(vtkRenderer* ren, vtkActor* actor,
@@ -383,10 +427,7 @@ void vtkOpenGLGlyph3DHelper::GlyphRenderInstances(vtkRenderer* ren, vtkActor* ac
     this->InstanceBuffersBuildTime.Modified();
   }
 
-  bool draw_surface_with_edges =
-    (actor->GetProperty()->GetEdgeVisibility() && representation == VTK_SURFACE);
-  for (int i = PrimitiveStart;
-       i < (draw_surface_with_edges ? PrimitiveEnd : PrimitiveTriStrips + 1); i++)
+  for (int i = PrimitiveStart; i < vtkOpenGLPolyDataMapper::PrimitiveEnd; i++)
   {
     this->DrawingVertices = i > PrimitiveTriStrips;
     if (this->Primitives[i].IBO->IndexCount)
@@ -474,9 +515,18 @@ void vtkOpenGLGlyph3DHelper::GlyphRenderInstances(vtkRenderer* ren, vtkActor* ac
               static_cast<GLsizei>(this->InstanceCulling->GetLOD(j).IBO->IndexCount),
               GL_UNSIGNED_INT, nullptr, this->InstanceCulling->GetLOD(j).NumberOfInstances);
 #else
-            glDrawElementsInstancedARB(mode,
-              static_cast<GLsizei>(this->InstanceCulling->GetLOD(j).IBO->IndexCount),
-              GL_UNSIGNED_INT, nullptr, this->InstanceCulling->GetLOD(j).NumberOfInstances);
+            if (GLAD_GL_ARB_draw_instanced)
+            {
+              glDrawElementsInstancedARB(mode,
+                static_cast<GLsizei>(this->InstanceCulling->GetLOD(j).IBO->IndexCount),
+                GL_UNSIGNED_INT, nullptr, this->InstanceCulling->GetLOD(j).NumberOfInstances);
+            }
+            else
+            {
+              glDrawElementsInstanced(mode,
+                static_cast<GLsizei>(this->InstanceCulling->GetLOD(j).IBO->IndexCount),
+                GL_UNSIGNED_INT, nullptr, this->InstanceCulling->GetLOD(j).NumberOfInstances);
+            }
 #endif
             this->InstanceCulling->GetLOD(j).IBO->Release();
           }
@@ -486,8 +536,16 @@ void vtkOpenGLGlyph3DHelper::GlyphRenderInstances(vtkRenderer* ren, vtkActor* ac
             glDrawArraysInstanced(
               GL_POINTS, 0, 1, this->InstanceCulling->GetLOD(j).NumberOfInstances);
 #else
-            glDrawArraysInstancedARB(
-              GL_POINTS, 0, 1, this->InstanceCulling->GetLOD(j).NumberOfInstances);
+            if (GLAD_GL_ARB_draw_instanced)
+            {
+              glDrawArraysInstancedARB(
+                GL_POINTS, 0, 1, this->InstanceCulling->GetLOD(j).NumberOfInstances);
+            }
+            else
+            {
+              glDrawArraysInstanced(
+                GL_POINTS, 0, 1, this->InstanceCulling->GetLOD(j).NumberOfInstances);
+            }
 #endif
           }
         }
@@ -547,8 +605,17 @@ void vtkOpenGLGlyph3DHelper::GlyphRenderInstances(vtkRenderer* ren, vtkActor* ac
         glDrawElementsInstanced(mode, static_cast<GLsizei>(this->Primitives[i].IBO->IndexCount),
           GL_UNSIGNED_INT, nullptr, numPts);
 #else
-        glDrawElementsInstancedARB(mode, static_cast<GLsizei>(this->Primitives[i].IBO->IndexCount),
-          GL_UNSIGNED_INT, nullptr, numPts);
+        if (GLAD_GL_ARB_draw_instanced)
+        {
+          glDrawElementsInstancedARB(mode,
+            static_cast<GLsizei>(this->Primitives[i].IBO->IndexCount), GL_UNSIGNED_INT, nullptr,
+            numPts);
+        }
+        else
+        {
+          glDrawElementsInstanced(mode, static_cast<GLsizei>(this->Primitives[i].IBO->IndexCount),
+            GL_UNSIGNED_INT, nullptr, numPts);
+        }
 #endif
 
         this->Primitives[i].IBO->Release();
